import es, gamethread, playerlib, popuplib, random
from configobj import ConfigObj

### Start Config

ITEMLASTS = 30 # How many seconds dropped items last, 0 = forever

SPAWNPROTECTIONTIME = 10 # How long spawn protection is
TSPAWNPROTECTIONCOLOR = [255, 0, 0, 255] # Red
CTSPAWNPROTECTIONCOLOR = [0, 0, 255, 255] # Blue

MAXHEALTH = 100
MAXPRIMARYAMMO = 1000
MAXSECONDARYAMMO = 500

class_command = 'class'
default_t_class = 'T Medic (MP5Navy)'
default_ct_class = 'CT Medic (MP5Navy)'

WINTSOUND = 0 # An example would be 'wolfenstein/winct.mp3' which would be under ./cstrike/sound/
WINCTSOUND = 0

### End Config ###

### Globals ###

viewcone_userid = None

### Constants ###

FILEPATH = es.getAddonPath('wolfenstein') + '/'

### Loads ###

def load():
	es.server.queuecmd('est_enable_PerUserKeyEvents 1')
	if not es.exists('saycommand', class_command):
		es.regsaycmd(class_command, 'wolfenstein/class_say_command')

	for userid in es.getUseridList():
		wolfenstein.getPlayer(userid)

	if not es.exists('command', 'INTERNAL_USE_ONLY'):
		es.regcmd('INTERNAL_USE_ONLY', 'wolfenstein/internal')
	add_sounds()

def unload():
	es.server.queuecmd('est_enable_PerUserKeyEvents 0')
	es.addons.unregisterClientCommandFilter(_cc_filter)

	if es.exists('saycommand', class_command):
		es.unregsaycmd(class_command)

	for index in wolfenstein.indexes:
		gamethread.cancelDelayed('remove item %s'%index)

### Classes ###

class WolfensteinPlayer(object):
	def __init__(self, userid):
		self.userid = userid
		self.location = []

		self.drops = 0
		self.className = None
		self.classData = {}
		self.spawn = False

	def setAttributes(self):
		if not es.getplayerteam(self.userid) in [2, 3]:
			return

		if not self.className:
			es.sexec(self.userid, 'say %s'%class_command)
			return
		self.drops = 0

		player = playerlib.getPlayer(self.userid)
		if self.classData['model']:
			player.set('model', self.classData['model'])

		player.set('health', self.classData['health'])
		player.set('speed', self.classData['speed'])

		es.server.queuecmd('es_xgive %s player_weaponstrip'%self.userid)
		es.server.queuecmd('es_xfire %s player_weaponstrip strip'%self.userid)

		es.delayed(0.5, 'es_xgive %s weapon_knife'%self.userid)
		es.delayed(1, 'es_xfire %s player_weaponstrip kill'%self.userid)

		if self.classData['primary_weapon']:
			es.delayed(0, 'es_xgive %s weapon_%s'%(self.userid, self.classData['primary_weapon']))
			gamethread.delayed(0, es.delayed, (0, 'playerset ammo %s 1 %s'%(self.userid, self.classData['primary_ammo'])))

		if self.classData['secondary_weapon']:
			es.delayed(0, 'es_xgive %s weapon_%s'%(self.userid, self.classData['secondary_weapon']))
			gamethread.delayed(0, es.delayed, (0, 'playerset ammo %s 2 %s'%(self.userid, self.classData['secondary_ammo'])))

		if self.classData['grenades']:
			for grenade in self.classData['grenades'].split(','):
				es.delayed(0, 'es_xgive %s weapon_%s'%(self.userid, grenade))

		if self.classData['can_use_bomb']:
			if not player.get('c4'):
				es.delayed(0, 'es_xgive %s weapon_c4'%self.userid)

	def activateDrop(self):
		if self.classData['drop']:
			self.dropItem(self.classData['drop'])

	def dropItem(self, item):
		if self.drops >= self.classData['drop_limit']:
			es.tell(self.userid, '#multi', '#lightgreen[Wolfenstein]: #defaultYou have already used up all your drops!')
			return
		self.drops += 1

		x, y, z = playerlib.getPlayer(self.userid).get('viewangle')
		model = 'items/357ammobox' if item == 'Ammo Pack' else 'items/healthkit'

		es.server.queuecmd('es_xsetang %s 40 %s 0'%(self.userid, y + 180))
		es.server.queuecmd('es_prop_physics_create %s %s;es_xset wolfenstein_type %s;es_xset wolfenstein_userid %s;es_doblock wolfenstein/dropItem'%(self.userid, model, 'health' if item == 'Medpack' else 'ammo', self.userid))
		es.server.queuecmd('es_xsetang %s %s %s %s'%(self.userid, x, y, z))

	def changeClass(self, choice):
		self.className = choice
		self.classData = wolfenstein.generateData(wolfenstein.classData[choice])
		es.tell(self.userid, '#multi', '#lightgreen[Wolfenstein]: #defaultYou are now a #lightgreen%s#default!'%choice)

	def spawnProtection(self):
		if not SPAWNPROTECTIONTIME:
			return

		if not es.getplayerteam(self.userid) in [2, 3]:
			return
		self.spawn = True

		gamethread.cancelDelayed('protection %s'%self.userid)
		gamethread.delayedname(SPAWNPROTECTIONTIME, 'protection %s'%self.userid, self.stopSpawnProtection)

		player = playerlib.getPlayer(self.userid)
		player.set('health', 1125)
		player.set('color', TSPAWNPROTECTIONCOLOR if es.getplayerteam(self.userid) == 2 else CTSPAWNPROTECTIONCOLOR)

	def stopSpawnProtection(self):
		self.spawn = False
		player = playerlib.getPlayer(self.userid)

		player.set('color', [255, 255, 255, 255])
		if not self.className:
			player.set('health', 100)
			return
		player.set('health', self.classData['health'])

class PlayerManager(object):
	def __init__(self):
		self.players = {}
		self.indexes = {}

		self.classData = dict(ConfigObj(FILEPATH + 'wolfenstein.ini'))

	def getPlayer(self, userid):
		userid = int(userid)
		if not userid in self.players:
			self.players[userid] = WolfensteinPlayer(userid)
		return self.players[userid]

	def delPlayer(self, userid):
		if int(userid) in self.players:
			del self.players[int(userid)]

	def generateData(self, data):
		temp = {}
		for key in data:
			value = data[key]
			if '-' in value and (value.count('-') > 1 or not value[0] == '-'):
				temp[key] = value
				continue

			if '.' in value and value.count('.') == 1 and value.replace('-', '').replace('.', '').isdigit():
				temp[key] = float(value)
				continue

			if value.isdigit():
				temp[key] = int(value)
				continue
			temp[key] = value
		return temp

wolfenstein = PlayerManager()

### Events ###

def es_map_start(ev):
	add_sounds()

def add_sounds():
	for sound in [WINTSOUND, WINCTSOUND]:
		if sound:
			if os.path.isfile(str(es.ServerVar('eventscripts_gamedir')) + '/sound/' + sound):
				es.stringtable('downloadables', sound)

def player_hurt(ev):
	if wolfenstein.getPlayer(ev['userid']).spawn:
		playerlib.getPlayer(ev['userid']).set('health', 1125)

def round_end(ev):
	for userid in es.getUseridList():
		gamethread.cancelDelayed('protection %s'%userid)

	sound = WINTSOUND if ev['winner'] == '2' else WINCTSOUND
	if not sound:
		return

	for userid in es.getUseridList():
		es.cexec(userid, 'play %s'%sound)

def player_activate(ev):
	wolfenstein.getPlayer(ev['userid'])

def player_team(ev):
	if ev['team'] in '23':
		classmenu_select(ev['userid'], default_t_class if es.getplayerteam(ev['userid']) == 2 else default_ct_class, None)
		es.sexec(ev['userid'], 'say %s'%class_command)

def player_spawn(ev):
	wolfenstein.getPlayer(ev['userid']).setAttributes()
	wolfenstein.getPlayer(ev['userid']).spawnProtection()

def player_death(ev):
	userid = int(ev['userid'])
	if userid in classes:
		wolfenstein.getPlayer(userid).changeClass(classes[userid])
		del classes[userid]
	gamethread.cancelDelayed('protection %s'%userid)

def player_disconnect(ev):
	wolfenstein.delPlayer(ev['userid'])

def bomb_beginplant(ev):
	userid = ev['userid']
	if not wolfenstein.getPlayer(userid).classData['can_use_bomb']:
		es.setplayerprop(userid, 'CCSPlayer.baseclass.localdata.m_vecBaseVelocity', ','.join([str(random.randint(-400, 400)) for x in '123']))

def bomb_begindefuse(ev):
	userid = ev['userid']
	if not wolfenstein.getPlayer(userid).classData['can_use_bomb']:
		es.setplayerprop(userid, 'CCSPlayer.baseclass.localdata.m_vecBaseVelocity', ','.join([str(random.randint(-400, 400)) for x in '123']))

def round_start(ev):
	es.msg('#multi', '#lightgreen[Wolfenstein]: #defaultType #lightgreen%s#default to change your class!'%class_command)
	for index in wolfenstein.indexes:
		gamethread.cancelDelayed('remove item %s'%index)
	wolfenstein.indexes.clear()

	'''
	userid = es.getuserid()
	if userid:
		es.server.queuecmd('es_xfire %s weapon_c4 kill'%userid)
		userids = filter(lambda x: wolfenstein.getPlayer(x).classData['can_use_bomb'], playerlib.getUseridList('#t'))

		if not userids:
			return
		es.delayed(0.1, 'es_xgive %s weapon_c4'%random.choice(userids))
	'''

def est_client_keypress(ev):
	if not ev['keyname'] == 'attack2':
		return

	userid = ev['userid']
	if ev['status'] == '0':
		return
	wolfenstein.getPlayer(userid).activateDrop()

### Commands ###

def class_say_command():
	userid = es.getcmduserid()
	if popuplib.isqueued('Class Menu', userid):
		popuplib.close('Class Menu', userid)

	class_menu = popuplib.easymenu('Class Menu', '_popup_choice', classmenu_select)
	team = 'T' if es.getplayerteam(userid) == 2 else 'CT'

	for playerClass in wolfenstein.classData:
		if wolfenstein.classData[playerClass]['restriction']:
			if classCount(playerClass) >= wolfenstein.classData[playerClass]['restriction']:
				continue

		if not team == wolfenstein.classData[playerClass]['team']:
			continue
		class_menu.addoption(playerClass, playerClass)
	popuplib.send('Class Menu', userid)

def _cc_filter(userid, args):
	if args[0].lower() in ['drop', 'buy']:
		return False
	return True
es.addons.registerClientCommandFilter(_cc_filter)

def internal():
	args = es.getargs().split(' ')
	if not len(args) == 1:
		return

	index = int(args[0])
	for userid in es.getUseridList():
		player = playerlib.getPlayer(userid)
		if player.get('color') == (2, 4, 8, 255):
			player.set('color', [255, 255, 255, 255])
			break

	if not index in wolfenstein.indexes:
		return
	attributes = wolfenstein.indexes[index]

	if attributes[0] == 'health':
		player.set('health', min(max(player.get('health'), MAXHEALTH), player.get('health') + attributes[1]))
		return

	if player.get('primary'):
		player.set('ammo', ['1', min(MAXPRIMARYAMMO, player.get('ammo', '1') + attributes[1])])

	if player.get('secondary'):
		player.set('ammo', ['2', min(MAXSECONDARYAMMO, player.get('ammo', '2') + attributes[1])])

	gamethread.cancelDelayed('remove item %s'%index)
	del wolfenstein.indexes[index]

### Popups ###

classes = {}
def classmenu_select(userid, choice, popupid):
	setAttributes = False
	if not playerlib.getPlayer(userid).get('isdead') and wolfenstein.getPlayer(userid).className:
		es.tell(userid, '#multi', '#lightgreen[Wolfenstein]: #defaultYour class will be changed to #lightgreen%s#default when you die'%choice)
		classes[userid] = choice
		return

	if not wolfenstein.getPlayer(userid).className:
		setAttributes = True
	wolfenstein.getPlayer(userid).changeClass(choice)

	if setAttributes:
		wolfenstein.getPlayer(userid).setAttributes()

### Misc ###

def classCount(pc):
	count = 0
	for userid in es.getUseridList():
		if wolfenstein.getPlayer(userid).className == pc:
			count += 1
	return count

def dropItem():
	userid = int(es.ServerVar('wolfenstein_userid'))
	index = int(es.ServerVar('eventscripts_lastgive'))

	if not es.createentitylist('point_servercommand'):
		es.server.queuecmd('est_banned_entities ""')
		es.server.queuecmd('es_xgive %s point_servercommand'%userid)

	es.server.queuecmd('est_setentname %s drop_%s'%(index, userid))
	es.server.queuecmd('es_xfire %s drop_%s addoutput "spawnflags 32"'%(userid, userid))
	es.server.queuecmd('es_xfire %s drop_%s addoutput "OnBreak !activator,color,2 4 8"'%(userid, userid))
	es.server.queuecmd('es_xfire %s drop_%s addoutput "OnBreak point_servercommand,command,INTERNAL_USE_ONLY %s"'%(userid, userid, index))

	wolfenstein.indexes[index] = [str(es.ServerVar('wolfenstein_type')), wolfenstein.getPlayer(userid).classData['drop_amount']]
	es.server.queuecmd('est_touch_trigger %s player'%index)

	if ITEMLASTS:
		gamethread.delayedname(ITEMLASTS, 'remove item %s'%index, removeItem, index)

def removeItem(index):
	if not index in wolfenstein.indexes:
		return

	del wolfenstein.indexes[index]
	if index in es.createentitylist():
		es.server.queuecmd('es_xremove %s'%index)
